@@ -6,25 +6,16 @@
  * \ingroup bke
  */
 
-#include "MEM_guardedalloc.h"
-
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_modifier_types.h"
-#include "DNA_scene_types.h"
+#include "DNA_object_types.h"
 
-#include "BKE_customdata.h"
-#include "BKE_lib_id.h"
+#include "BKE_customdata.hh"
 #include "BKE_mesh.hh"
-#include "BKE_mesh_runtime.hh"
-#include "BKE_modifier.h"
 #include "BKE_multires.hh"
-#include "BKE_subdiv.hh"
-#include "BKE_subsurf.hh"
 #include "BLI_math_vector.h"
 
-#include "DEG_depsgraph_query.hh"
-
 #include "multires_reshape.hh"
 
 static void multires_subdivide_create_object_space_linear_grids(Mesh *mesh)
@@ -36,7 +27,7 @@ static void multires_subdivide_create_object_space_linear_grids(Mesh *mesh)
   const blender::Span<int> corner_verts = mesh->corner_verts();
 
   MDisps *mdisps = static_cast<MDisps *>(
-      CustomData_get_layer_for_write(&mesh->loop_data, CD_MDISPS, mesh->totloop));
+      CustomData_get_layer_for_write(&mesh->corner_data, CD_MDISPS, mesh->corners_num));
   for (const int p : faces.index_range()) {
     const blender::IndexRange face = faces[p];
     const float3 face_center = mesh::face_center_calc(positions, corner_verts.slice(face));
@@ -72,9 +63,10 @@ void multires_subdivide_create_tangent_displacement_linear_grids(Object *object,
 
   const int new_top_level = mmd->totlvl + 1;
 
-  const bool has_mdisps = CustomData_has_layer(&coarse_mesh->loop_data, CD_MDISPS);
+  const bool has_mdisps = CustomData_has_layer(&coarse_mesh->corner_data, CD_MDISPS);
   if (!has_mdisps) {
-    CustomData_add_layer(&coarse_mesh->loop_data, CD_MDISPS, CD_SET_DEFAULT, coarse_mesh->totloop);
+    CustomData_add_layer(
+        &coarse_mesh->corner_data, CD_MDISPS, CD_SET_DEFAULT, coarse_mesh->corners_num);
   }
 
   if (new_top_level == 1) {
